'use strict'

// Imports
const env = require("../toolbox").env
const utils = require("../toolbox").utils;
const puppeteer = require('puppeteer');
const Queue = require("./queue")
const toMatchImageSnapshot = require('jest-image-snapshot').toMatchImageSnapshot;
const expect = require('expect')

// Set snapshot jest
expect.extend({ toMatchImageSnapshot });


class Browser {

    constructor() {
        this.browser = null
    }

    async execute(queue) {
        if (queue === null || queue === undefined || !(queue instanceof Queue)) {
            throw "Queue is not valid"
        }
        if (this.browser === null) {
            this.browser = await puppeteer.launch(env.config);
            let pages = await this.browser.pages();
            await Promise.all(pages.map(async page => await page.close()));
            this.browser.on('targetdestroyed', async target => {
                const openPages = await this.browser.pages();
                if (openPages.length == 0) {
                    await this.browser.close();
                }
            });
        }
        await Promise.all(queue.methods.map(async config => {
            let page = await this.browser.newPage();
            page.on('console', msg => {
                for (let i = 0; i < msg.args().length; ++i)
                    console.log(`${i}: ${msg.args()[i]}`);
            });
            try {
                let path = utils.screenshotDir(config.url)
                let firstImage = path + "snap" + config.picName + ".png"
                let secondImage = path + "comp" + config.picName + ".png"
                if (!fs.existsSync(firstImage)) {
                    env.moduleSetup(config.setup)
                    await page.emulate(config.version)
                    await utils.goto(page, config.url)
                    await utils.reload(page)
                    await utils.scrollPage(page)
                    console.log("Taking a picture")
                    await page.screenshot({
                        path: firstImage, 
                        fullPage: true
                    });
                } else {
                    console.log("Taking a second picture")
                    let image = await page.screenshot({
                        path: secondImage, 
                        fullPage: true
                    });
                    console.log("Comparing the picture")
                    await expect(image).toMatchImageSnapshot({
                        failureThreshold: env.threshold,
                        failureThresholdType: 'percent',
                        updatePassedSnapshot: true,
                        customSnapshotsDir: path,
                        customSnapshotIdentifier: "snap"
                    })
                }
            } catch (e) {}
             finally {
                await page.close()
             }
        }))
    }
}

module.exports = Browser